@@ -11,8 +11,7 @@
 #ifndef _VZCTL_H_
 #define _VZCTL_H_
 
-#define	VERSION			"2.7.0-20"
-#define MIN_NUMIPTENT		16
+#define	VERSION			"2.7.0-21"
 
 /* Action numeration should begin with 1. Value 0 is reserved. */
 #define ACTION_CREATE		1
@@ -45,8 +44,6 @@
 #define UNLCPUUNITS             1000
 #define HNCPUUNITS              1000
 
-#define DIST_DIR		"/etc/sysconfig/vz-scripts/dists"
-
 /* setmode flags */
 enum {
         SET_NONE = 0,
