@@ -276,7 +276,7 @@ static int parse_setmode(vps_param *vps_p, char *val)
 	return 0;
 }
 
-int conf_parse_strlist(list_head_t *list, char *val, int checkdup)
+int conf_parse_strlist(list_head_t *list, const char *val, int checkdup)
 {
 	if (add_str2list(list, val))
 		return ERR_NOMEM;
@@ -1164,8 +1164,11 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 		ret = conf_parse_str(&vps_p->res.tmpl.def_ostmpl, val, 1);
 		break;
 	case PARAM_PKGSET:
-		logger(0, 0, "Warning: option --pkgset is not supported"
-			" --ostemplate will be used");
+		ret = conf_parse_str(&vps_p->res.tmpl.pkgset, val, 1);
+		break;
+	case PARAM_PKGVER:
+		ret = conf_parse_str(&vps_p->res.tmpl.pkgver, val, 1);
+		break;
 	case PARAM_OSTEMPLATE:
 		ret = conf_parse_str(&vps_p->res.tmpl.ostmpl, val, 1);
 		break;
@@ -1450,6 +1453,32 @@ int vps_save_config(envid_t veid, char *path, vps_param *new_p,
 	return 0;
 }
 
+int vps_remove_cfg_param(envid_t veid, char *path, char *name)
+{
+	list_head_t conf;
+	conf_struct *line;
+	int ret, found;
+
+	list_head_init(&conf);
+	if ((ret = read_conf(path, &conf)))
+		return ret;
+	if (list_empty(&conf))
+		return 0;
+	found = 0;
+	while ((line = find_conf_line(&conf, name, '=')) != NULL) {
+		free(line->val);
+		list_del(&line->list);
+		free(line);
+		found++;
+	}
+	if (found)
+		write_conf(path, &conf);	
+	free_str_param(&conf);
+
+	return 0;
+
+}
+
 /********************************************************************/
 int vps_parse_opt(envid_t veid, vps_param *param, int opt, const char *rval,
 	struct mod_action *action)
