@@ -332,7 +332,7 @@ int vps_net_ctl(vps_handler *h, envid_t veid, int op, net_param *net,
 
 	if (list_empty(ip_h) &&	!net->delall) {
 		/* make initial network setup on VPS start*/
-		if (state == STATE_STARTING)
+		if (state == STATE_STARTING && op == ADD)
 			vps_ip_configure(h, veid, actions, root, op, net,state);
 		return 0;
 	}
