@@ -119,8 +119,7 @@ int make_dir(char *path, int full)
 		return 0;
 	if (!stat_file(path)) {
 		if (mkdir(path, 0755)) {
-			logger(0, errno, "Can't create directory %s",
-				path);
+			logger(0, errno, "Can't create directory %s", path);
 			return 1;
 		}
 	}
@@ -197,11 +196,15 @@ int cp_file(char *dst, char *src)
 		if (!ret)
 			break;
 		else if (ret < 0) {
-			logger(0, errno, "Unable to read from %s", dst);
+			logger(0, errno, "Unable to read from %s", src);
+			ret = -1;
+			break;
+		}
+		if (write(fd_dst, buf, len) < 0) {
+			logger(0, errno, "Unable write to %s", dst);
 			ret = -1;
 			break;
 		}
-		write(fd_dst, buf, len);
 	}
 	close(fd_src);
 	close(fd_dst);
@@ -442,6 +445,12 @@ const char *get_vps_state_str(int vps_state)
 	case STATE_STARTING:
 		p = "starting";
 		break;
+	case STATE_STOPPED:
+		p = "stopped";
+		break;
+	case STATE_STOPPING:
+		p = "stopping";
+		break;
 	}
 	return p;
 }
