@@ -27,42 +27,13 @@
 #include "exec.h"
 #include "config.h"
 #include "vzerror.h"
-#include "types.h"
+#include "create.h"
 #include "util.h"
 #include "lock.h"
 #include "modules.h"
 
-
 extern struct mod_action g_action;
-
-static void fix_numiptent(ub_param *ub)
-{
-	unsigned long min_ipt;
-        ub_res *res;
-
-	res = get_ub_res(ub, UB_IPTENTRIES);
-	if (res == NULL)
-		return;
-	min_ipt = min_ul(res->limit[0], res->limit[1]);
-        if (min_ipt < MIN_NUMIPTENT) {
-		logger(0, 0, "Warning: NUMIPTENT %d:%d is less"
-			" than minimally allowable value, set to %d:%d",
-                        res->limit[0], res->limit[1],
-                        MIN_NUMIPTENT, MIN_NUMIPTENT);
-		res->limit[0] = MIN_NUMIPTENT;
-		res->limit[1] = MIN_NUMIPTENT;
-	}
-}
-
-static void fix_cpu(cpu_param *cpu)
-{
-	unsigned long *units;
-
-	if (cpu->units == NULL && cpu->weight == NULL) {
-		units = malloc(sizeof(*units));
-		*cpu->units = UNLCPUUNITS;
-	}
-}
+extern int do_enter(vps_handler *h, envid_t veid, char *root);
 
 int parse_opt(envid_t veid, int argc, char *argv[], struct option *opt,
 	vps_param *param)
@@ -145,8 +116,6 @@ static int start(vps_handler *h, envid_t veid, vps_param *g_p,
 	vps_param *cmd_p)
 {
 	int ret;
-	char *dist_name;
-	dist_actions *actions = NULL;
 
 	if ((ret = check_ub(&g_p->res.ub)))
 		return ret;
@@ -160,19 +129,8 @@ static int start(vps_handler *h, envid_t veid, vps_param *g_p,
 		logger(0, 0, "VPS is already running");
 		return VZ_VE_RUNNING;
 	}
-	actions = malloc(sizeof(*actions));
-	dist_name = get_dist_name(&g_p->res.tmpl);
-	if ((ret = read_dist_actions(dist_name, DIST_DIR, actions)))
-               	return ret;
-	if (dist_name != NULL)
-		free(dist_name);
-	fix_numiptent(&g_p->res.ub);
-	fix_cpu(&g_p->res.cpu);
-	ret = vps_start(h, veid, g_p, actions,
-		cmd_p->opt.skip_setup == YES ? SKIP_SETUP : 0,
-		&g_action);
-	free_dist_actions(actions);
-
+	ret = vps_start(h, veid, g_p,
+		cmd_p->opt.skip_setup == YES ? SKIP_SETUP : 0, &g_action);
 	return ret;
 }
 
@@ -231,6 +189,7 @@ static int parse_create_opt(envid_t veid, int argc, char **argv,
 	vps_param *param)
 {
 	int ret;
+	struct option *opt;
 	struct option create_options[] = {
 	{"ostemplate",	required_argument, NULL, PARAM_OSTEMPLATE},
 	{"pkgver",      required_argument, NULL, PARAM_PKGVER},
@@ -242,7 +201,12 @@ static int parse_create_opt(envid_t veid, int argc, char **argv,
 	{"hostname",	required_argument, NULL, PARAM_HOSTNAME},
 	{ NULL, 0, NULL, 0 }
 };
-	ret = parse_opt(veid, argc, argv, create_options, param);
+	
+	opt = mod_make_opt(create_options, &g_action, NULL);
+	if (opt == NULL)
+		return -1;
+	ret = parse_opt(veid, argc, argv, opt, param);
+	free(opt);
 
 	return ret;
 }
@@ -251,66 +215,9 @@ static int create(vps_handler *h, envid_t veid, vps_param *vps_p,
 	vps_param *cmd_p)
 {
 	int ret;
-	char src[STR_SIZE];
-	char dst[STR_SIZE];
-	const char *sample_config = NULL;
 
-	get_vps_conf_path(veid, dst, sizeof(dst));
-	if (stat_file(dst))
-		sample_config = NULL;
-	else if (cmd_p->opt.config != NULL) {
-		snprintf(src, sizeof(src),  VPS_CONF_DIR "ve-%s.conf-sample",
-			cmd_p->opt.config);
-		if (!stat_file(src)) {
-			logger(0, 0, "File %s is not found", src);
-			return VZ_CP_CONFIG;
-		}
-		sample_config = cmd_p->opt.config;
-	} else if (vps_p->opt.config != NULL) {
-		snprintf(src, sizeof(src),  VPS_CONF_DIR "ve-%s.conf-sample",
-			vps_p->opt.config);
-		if (stat_file(src)) 
-			sample_config = vps_p->opt.config;
-	}
-	if (sample_config != NULL) {
-		vps_param *conf_p;
+	ret = vps_create(h, veid, vps_p, cmd_p, &g_action);
 
-		if ((conf_p = init_vps_param()) == NULL)
-			return -1;
-		vps_parse_config(veid, src, conf_p, NULL);
-		merge_vps_param(vps_p, conf_p);
-		if (conf_p->opt.origin_sample == NULL)
-			cmd_p->opt.origin_sample = strdup(sample_config);
-		free_vps_param(conf_p);
-	}
-	/* Set default ostemplate */
-	if (cmd_p->res.tmpl.ostmpl == NULL) {
-		if (vps_p->res.tmpl.ostmpl != NULL) {
-			cmd_p->res.tmpl.ostmpl =
-				strdup(vps_p->res.tmpl.ostmpl);
-		} else if (vps_p->res.tmpl.def_ostmpl != NULL) {
-			cmd_p->res.tmpl.ostmpl =
-				strdup(vps_p->res.tmpl.def_ostmpl);
-		}
-	}
-	merge_vps_param(vps_p, cmd_p);
-	if ((ret = vps_create(h, veid, &vps_p->res.fs, &vps_p->res.tmpl)))
-		return ret;
-	vps_postcreate(veid, &vps_p->res.fs, &vps_p->res.tmpl);
-	if (sample_config != NULL)
-		cp_file(dst, src);
-	/* store root, private, ostemplate in case default used */
-	if (cmd_p->res.fs.root_orig == NULL &&
-		vps_p->res.fs.root_orig != NULL)
-	{
-		cmd_p->res.fs.root_orig = strdup(vps_p->res.fs.root_orig);
-	}
-	if (cmd_p->res.fs.private_orig == NULL &&
-		vps_p->res.fs.private_orig != NULL)
-	{
-		cmd_p->res.fs.private_orig = strdup(vps_p->res.fs.private_orig);
-	}
-	vps_save_config(veid, dst, cmd_p, vps_p, NULL);
 	return ret;
 }
 
@@ -396,7 +303,7 @@ static int parse_set_opt(envid_t veid, int argc, char *argv[],
 	int ret;
 	struct option *opt;
 	
-	opt = mod_make_opt(get_set_opt(), &g_action);
+	opt = mod_make_opt(get_set_opt(), &g_action, NULL);
 	if (opt == NULL)
 		return -1;
 	ret = parse_opt(veid, argc, argv, opt, param);
@@ -413,6 +320,8 @@ static int set(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *vps_p,
 	char *dist_name;
 
 	ret = 0;
+
+	cmd_p->g_param = g_p;
 	if (cmd_p->opt.setmode == SET_RESTART && !cmd_p->opt.save) {
 		logger(0, 0, "it's impossible to use"
 			" restart mode without --save");
@@ -447,7 +356,7 @@ static int set(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *vps_p,
 	/* Setup password */
 	if (!list_empty(&cmd_p->res.misc.userpw)) {
 		if (!is_run)
-			if ((ret = vps_start(h, veid, g_p, NULL,
+			if ((ret = vps_start(h, veid, g_p,
 				SKIP_SETUP|SKIP_ACTION_SCRIPT, NULL)))
 			{
 				goto err;
@@ -468,8 +377,11 @@ static int set(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *vps_p,
 		}
 	}
 	/* Skip apply parameters on stopped VPS */
-	if (cmd_p->opt.save && !is_run)
-		return 0;
+	if (cmd_p->opt.save && !is_run) {
+		ret = mod_setup(h, veid, STATE_STOPPED, SKIP_NONE, &g_action,
+			cmd_p);
+		return ret;
+	}
 	if (is_run) {
 		ret = check_set_mode(h, veid, cmd_p->opt.setmode,
 			cmd_p->opt.save, &cmd_p->res, &g_p->res);
@@ -508,12 +420,7 @@ static int umount(vps_handler *h, envid_t veid, vps_param *g_p,
 static int enter(vps_handler *h, envid_t veid, vps_param *g_p,
 	vps_param *cmd_p)
 {
-	int i, ret;
-	char term[64];
-	char *p;
 	char *root = g_p->res.fs.root;
-	char *arg[] = {"bash",  "-i", NULL};
-	char *env[] = {"HOME=/", "PATH=/bin:/sbin:/usr/bin:/usr/sbin:", NULL, NULL};
 
 	set_log_file(NULL);
 	if (check_var(root, "VE_ROOT is not set"))
@@ -522,25 +429,7 @@ static int enter(vps_handler *h, envid_t veid, vps_param *g_p,
 		logger(0, 0, "VPS is not running");
 		return VZ_VE_NOT_RUNNING;
 	}
-	p = getenv("TERM");
-	snprintf(term , sizeof(term), "TERM=%s", p);
-	env[2] = term;
-	if ((ret = vz_chroot(root)))
-		return ret;
-	if ((ret = vz_setluid(veid)))
-		return ret;
-	ret = vz_env_create_ioctl(h, veid, VE_ENTER);
-       	if (ret != veid) {
-		logger(0, 0, "enter: environment is not changed\n");
-		return VZ_ENVCREATE_ERROR;
-       	}
-	for (i = 3; i < FOPEN_MAX; i++)
-		close(i);
-
-	execve("/bin/bash", arg, env);
-	execve("/bin/sh", arg, env);
-	logger(0, errno, "enter: unable to exec bash");
-
+	do_enter(h, veid, root);
 	return 0;
 }
 
@@ -599,13 +488,23 @@ static int show_status(vps_handler *h, envid_t veid, vps_param *param)
 	return 0;
 }
 
-static int parse_custom_opt(int argc, char **argv)
+static int parse_custom_opt(envid_t veid, int argc, char **argv,
+	vps_param *param, const char *name)
 {
-	return 0;
+	int ret;
+	struct option *opt;
+	
+	opt = mod_make_opt(NULL, &g_action, name);
+	if (opt == NULL)
+		return -1;
+	ret = parse_opt(veid, argc, argv, opt, param);
+	free(opt);
+
+	return ret;
 }
 
 int parse_action_opt(envid_t veid, int action, int argc, char *argv[],
-	vps_param *param)
+	vps_param *param, const char *name)
 {
 	int ret = 0;
 
@@ -629,14 +528,19 @@ int parse_action_opt(envid_t veid, int action, int argc, char *argv[],
 	case ACTION_EXEC:
 	case ACTION_EXEC2:
 	case ACTION_EXEC3:
-	case ACTION_RUNSCRIPT:
     		if (argc < 2) {
 			fprintf(stderr, "No command line given for exec\n");
 			ret = VZ_INVALID_PARAMETER_SYNTAX;
 		}
 		break;
+	case ACTION_RUNSCRIPT:
+    		if (argc < 2) {
+			fprintf(stderr, "Script does not specified\n");
+			ret = VZ_INVALID_PARAMETER_SYNTAX;
+		}
+		break;
 	case ACTION_CUSTOM:
-		ret = parse_custom_opt(argc, argv);
+		ret = parse_custom_opt(veid, argc, argv, param, name);
 		break;
 	default :
 		if ((argc - 1) > 0) {
@@ -718,9 +622,11 @@ int run_action(envid_t veid, int action, vps_param *g_p, vps_param *vps_p,
 			vps_save_config(veid, fname, cmd_p, vps_p, &g_action);
 			logger(0, 0, "Saved parameters for VPS %d", veid);
 		} else {
-			logger(0, 0, "WARNING: Settings were not saved"
+			if (list_empty(&cmd_p->res.misc.userpw)) {
+				logger(0, 0, "WARNING: Settings were not saved"
 				" and will be resetted to original values on"
 				" next start (use --save flag)");
+			}
 		}
 		break;
 	case ACTION_STATUS:
@@ -736,6 +642,12 @@ int run_action(envid_t veid, int action, vps_param *g_p, vps_param *vps_p,
 		if (ret && action == ACTION_EXEC)
 			ret = VZ_COMMAND_EXECUTION_ERROR;
 		break;
+	case ACTION_RUNSCRIPT:
+		ret = vps_run_script(h, veid, argv[0], g_p);
+		break;
+	case ACTION_CUSTOM:
+		ret = mod_setup(h, veid, 0, 0, &g_action, g_p);
+		break;
 	}
 err:
 	/* Unlock VPS in case lock taken */
